/*
 * Copyright (c) 2000, 2012 Martin J. Mohlenkamp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/** \file tfast1.c

\brief Quick test file that the fast1 transform is working correctly.

Please see README for directions to the main files to look at. 

This test program has not been documented for other users.

*/


#include "../libftsh.h"
/*************************************************************************/
void main(int argc,
          char *argv[]
          ){

  /* initialize:
    coefs_inrow
    **/

  register int i,m;
  int numpts,band_limit,order_limit,halfpts;
  REAL g_epsilon,epsilon;
  FILE *fp;
  REAL *gnode,*gweight;
  char out_file[40];
  REAL ratio;

  /* additional for synthesis */
  Pm_1D *pmn_matrix;
  REAL norm;
  time_t now;
  REAL **hat;
  REAL **m_data;
  Pm_1D_Workspace pmnwork;
  char prec_file[40];

  REAL **hat2,error;

  /*************************************************************************/
  /* set parameters ***/

  /* numpts is the number of sample points */
  numpts=8;
  if(getoption("-n",argc,argv)!=NULL) 
    numpts=atoi(getoption("-n",argc,argv));

  /* g_epsilon is the target accuracy for the gaussian nodes */
#if USE_double
  g_epsilon=1e-15;
#else
  g_epsilon=1e-5;
#endif
  if(getoption("-ge",argc,argv)!=NULL) 
    g_epsilon=atof(getoption("-ge",argc,argv));

  /* epsilon is the truncation threshold and target accuracy*/
#if USE_double
  epsilon=1e-12;
#else
  epsilon=1e-5;
#endif
  if(getoption("-e",argc,argv)!=NULL) 
    epsilon=atof(getoption("-e",argc,argv));

  /* band_limit is the band limit, which restricts n < band_limit */
  band_limit=numpts-1;
  if(getoption("-bl",argc,argv)!=NULL) 
    band_limit=atoi(getoption("-bl",argc,argv));

  /* We can only resolve bandlimit < numpts.*/
  /* ?? therefore we lose at least one */
  assert(band_limit>0);
  assert(band_limit<numpts);

  /* should always be the same */
  order_limit=band_limit;

  /* file to put initialization stuff in */
  strncpy(out_file,"ftshout1.bin",40);
  if(getoption("-f",argc,argv)!=NULL) 
   strncpy(out_file,getoption("-f",argc,argv),40);

  /* print parameters */
  printf("#numpts=%i epsilon=%e g_epsilon=%e band_limit=%i\n",
	 numpts,epsilon,g_epsilon,band_limit);

  /*************************************************************************/
  /*************************************************************************/
  /* construct the gaussian nodes and weights */

  /* get memory */
  
  /* for the nodes */
  gnode=(REAL *)calloc(numpts,sizeof(REAL));
  assert(gnode!=NULL);

  /* for the weights */
  gweight=(REAL *)calloc(numpts,sizeof(REAL));
  assert(gweight!=NULL);

  /* actually make them */
  mk_gnode_gweight(gnode,gweight,g_epsilon,numpts);

  /*************************************************************************/
  /*************************************************************************/

  /* open file*/
  fp=fopen(out_file,"wb");

  /*************************************************************************/

  /* initialize, dump to file, and clean up */
  ratio=fast1_tofile(numpts,band_limit,gnode,gweight,epsilon,fp);

  printf("compression ratio = %5.2f\n",ratio);

  /*************************************************************************/
  /*************************************************************************/
  /* close file */
  fclose(fp);

  /* free nodes and weights */
  free(gnode);
  free(gweight);

  /*************************************************************************/
  /*** END of initialization phase *****************************************/
  /*************************************************************************/

  printf("end of initialization\n");

  /*************************************************************************/
  /*** BEGINNING of reverse (evaluation, synthesis) transform phase ********/
  /*************************************************************************/

  /********************************************************/
  /******* load from file *********************************/  

  /* set input file for precomputed stuff */
  /* ??? right type of copy ?? **/
  strncpy(prec_file,out_file,40);

  /* open file*/
#if 1
  fp=fopen(prec_file,"rb");
#else
  printf("opening file f1dat.bin\n");
  fp=fopen("f1dat.bin","rb");
#endif

  /* load precomputed initialization and set up workspace and such */
  fast1_fromfile(&numpts,&band_limit,&gnode,&gweight,
		 &pmn_matrix,&pmnwork,fp);

  /* close file */
  fclose(fp);
  /********************************************************/



  /* set order_limit */
  order_limit=band_limit; 

  /* need to decide of the structure for holding the coefficients.
     no need to fret too much, since the user will supply arrays for each
     m separately to the core (fast matrix apply) routine separately.
     for this diagnostic version, use whatever is easiest*/

  /**************************************************/
  /* get memory for hat, the transform coefficients */
  hat=(REAL **)calloc(2*order_limit-1,sizeof(REAL *));

  /* shift to center */
  hat+=order_limit-1;

  /* main memory: order_limit * order_limit*/
  hat[0]=(REAL *)calloc(order_limit*order_limit,sizeof(REAL));

  for(m=1;m<order_limit;m++){
    /* loop through other orders in positive/negative pairs */

    /* base +m on privious -m */
    hat[m]=hat[-(m-1)]+(order_limit -(m-1));
    
    /* base -m on +m */
    hat[-m]=hat[m]+(order_limit -m);
  }
  
  /* load in the coefficient values */

printf("made hat\n");

  /***********************************************************************/
  /* diagnostic: create values and normalize */
#if 1
  /* random seed */
  now = time(NULL);
  assert(now!=-1); /* -1 means can't read the time */
  srand((unsigned)((unsigned) now));
  
    /* fill data with randoms */
  for(i=0;i<order_limit*order_limit;i++){
    hat[0][i]=(2.0* rand()/(RAND_MAX + 1.0)) -1.0;
  }
  
  printf("random data ");
#else
    hat[0][0]=1.0;  printf("hat[0][0] data ");
  /*  hat[0][1]=1.0;  printf("hat[0][1] data ");*/
  /*hat[0][0]=1.0;  printf("hat[0][0] data ");*/
  
#endif

  norm=0.0;
  for(i=0;i<order_limit*order_limit;i++){
    /* accumulate norm */
      norm+=hat[0][i]*hat[0][i];
  }
  
  /* scale to norm 1*/
  norm=1.0/sqrt(norm);
  for(i=0;i<order_limit*order_limit;i++)
    hat[0][i]*=norm;

  printf("norm 1.\n");

#if 1
  /* diagnostic:
     save a copy of hat*/
  hat2=(REAL **)calloc(2*order_limit-1,sizeof(REAL *));
  hat2+=order_limit-1;
  hat2[0]=(REAL *)calloc(order_limit*order_limit,sizeof(REAL));
  for(m=1;m<order_limit;m++){
    hat2[m]=hat2[-(m-1)]+(order_limit -(m-1));
    hat2[-m]=hat2[m]+(order_limit -m);
  }
  for(i=0;i<order_limit*order_limit;i++)
    hat2[0][i]=hat[0][i];
#endif  


  /***********************************************************************/
  /* holder for the data after the ALF transform but before 
     longitude processing */
  m_data=(REAL **)calloc(2*order_limit-1,sizeof(REAL *));

  /* shift to center */
  m_data+=order_limit-1;

  /* main memory: (2*order_limit-1) * numpts */
  m_data[0]=(REAL *)calloc((2*order_limit-1)*numpts,sizeof(REAL));

  for(m=1;m<order_limit;m++){
    /* loop through other orders in positive/negative pairs */

    /* base +m on privious -m */
    m_data[m]=m_data[-(m-1)]+numpts;
    
    /* base -m on +m */
    m_data[-m]=m_data[m]+numpts;
  }

  /**********************************************************************/
  /**********************************************************************/
  /* If the pmn_matrix array has been constructed already,
     then this next bit does the transform in latitude. **/
  for(m=-order_limit+1;m<order_limit;m++){
    /* loop through the order */

    /* synthesize for that m */
    Pm_1d_synthesize(m_data[m],pmn_matrix+abs(m),hat[m],&pmnwork);

  } 
  /**********************************************************************/
  /**********************************************************************/

  /*************************************************************************/
  /* if this evaluator was a separate program we would dump:
     the function values
     perhaps some diagnostics
  /*************************************************************************/

  /*************************************************************************/
  /*** END of reverse (evaluation, synthesis) transform phase **************/
  /*************************************************************************/

  /* diagnostic: check the norm of the computed function.
     need: fcn
           gaussian weights (unmodified)
	   */
  norm=0.0;
  for(m=-order_limit+1;m<order_limit;m++){
    for(i=0;i<numpts;i++){
      norm+=gweight[i]*m_data[m][i]*m_data[m][i];
    }
  }
  printf("computed function norm %e\n",sqrt(norm));


  printf("end of synthesis\n");

  /*************************************************************************/
  /*** BEGINNING of forward (expansion, analysis) transform phase **********/
  /*************************************************************************/

  /***********************************************************/
  /* if this was a separate program, we would have to load in what was
     dumped by the initializer, as done in the synthesize step */

  /* load in the function values (already transformed in longitude */

  /**********************************************************************/
  /**********************************************************************/
  /* If the pmn_matrix array has been constructed already,
     then this next bit does the transform in latitude. **/
  for(m=-order_limit+1;m<order_limit;m++){
    /* loop through the order */
 
    /* analyse for that m */
    Pm_1d_analyse(hat[m],pmn_matrix+abs(m),m_data[m],&pmnwork);

  } 
  /**********************************************************************/
  /**********************************************************************/

 
  /*************************************************************************/
  /* If this expander was its own program, we would now dump:
     the computed coefficients
     perhaps some diagnostics
  /*************************************************************************/

  /*************************************************************************/
  /*** END of forward (expansion, analysis) transform phase ****************/
  /*************************************************************************/

  /* diagnostic: check the norm of the computed coefficients */
  norm=0;
  for(i=0;i<order_limit*order_limit;i++){
    /* accumulate norm */
      norm+=hat[0][i]*hat[0][i];
  }
  printf("computed coefficients have norm %e\n",sqrt(norm));

#if 1
  /* diagnostic: compute the error of the computed coefficients from the 
       origional coefficients */
  error=0.0;
  for(i=0;i<order_limit*order_limit;i++)
    error+=(hat2[0][i]-hat[0][i])*(hat2[0][i]-hat[0][i]);
  printf("computed coefficients error %e\n",sqrt(error));
#endif

  printf("end of analysis\n");
     
}
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
