/*
 * Copyright (c) 2000, 20 12 Martin J. Mohlenkamp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/** \file tgaus.c

\brief Check our Gaussian nodes and weights against an external routine.

This test program has not been documented for other users.
Please see README for direction to the right files to look at. 

This program requires SPHEREPACK3.0's file 'gaqd.f', available from
http://www2.cisl.ucar.edu/resources/legacy/spherepack
*/

#include "../libftsh.h"


void gaqd_(int *nlat,double *theta,double *wts,double *dwork,
      int *ldwork,int *ierror);

/*************************************************************************/
void main(int argc,
          char *argv[]
          ){

  register int i,j;
  int numpts,lwork,ierror;
  REAL *gnode, *gweight, *fcn;
  REAL *spgnode, *spgweight,*workspace;
  REAL epsilon,integral;
  FILE *fp;

  /* set parameters ***/

  /* numpts is the number of sample points */
  numpts=8;
  if(getoption("-n",argc,argv)!=NULL) 
    numpts=atoi(getoption("-n",argc,argv));

  /* epsilon is the target accuracy for the gaussian nodes */
#if USE_double
  epsilon=1e-15;
#else
  epsilon=1e-6;
#endif
  if(getoption("-e",argc,argv)!=NULL) 
    epsilon=atof(getoption("-e",argc,argv));

  /* print parameters */
  printf("#numpts=%i epsilon=%e\n",numpts,epsilon);
  
  /*************************************************************************/
  /* get memory */
  
  /* for the nodes */
  gnode=(REAL *)calloc(numpts,sizeof(REAL));
  assert(gnode!=NULL);

  /* for the weights */
  gweight=(REAL *)calloc(numpts,sizeof(REAL));
  assert(gweight!=NULL);

  /* for the test function */
  fcn=(REAL *)calloc(numpts,sizeof(REAL));
  assert(fcn!=NULL);

  /* for the nodes */
  spgnode=(REAL *)calloc(numpts,sizeof(REAL));
  assert(spgnode!=NULL);

  /* for the weights */
  spgweight=(REAL *)calloc(numpts,sizeof(REAL));
  assert(spgweight!=NULL);

  /* for spherepack workspace */
  lwork=numpts+1;
  workspace=(REAL *)calloc(lwork,sizeof(REAL));
  assert(workspace!=NULL);

  /*************************************************************************/
  /* construct the node and weights */

#if 0
  /* spherepack version */
  gaqd_(&numpts,spgnode,spgweight,workspace,&lwork,&ierror);
  if(ierror)
    fprintf(stderr,"ierror=%i\n",ierror);
#endif

  mk_gnode_gweight(gnode,gweight,epsilon,numpts);

  /*************************************************************************/
  /* output them **/
  
  fp=fopen("tgaus.dat","w");
  for(i=0;i<numpts;i++){
    fprintf(fp,"%e %e %e %e %e %e\n",
	    gnode[i],gweight[i],spgnode[i],spgweight[i],
	    gnode[i]-spgnode[i],gweight[i]-spgweight[i]);
  }

  /*************************************************************************/
  /* construct a test function at these points */

  for(i=0;i<numpts;i++){
#if 0
    fcn[i]=1.0;
#else
    fcn[i]=Pmncos(1,4,gnode[i])*Pmncos(1,3,gnode[i]);
#endif
  }

  /* integrate the function */
  integral=0.0;
  for(i=0;i<numpts;i++){
    integral+=fcn[i]*gweight[i];
  }
  printf("my integral=%e\n",integral);

#if 0
  /* integrate the function */
  integral=0.0;
  for(i=0;i<numpts;i++){
    integral+=fcn[i]*spgweight[i];
  }
  printf("spherepack integral=%e\n",integral);
#endif



}
/*************************************************************************/


