/*
 * Copyright (c) 2000, 2012 Martin J. Mohlenkamp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "libftsh.h"

#ifndef ENTEREXIT
#define ENTEREXIT 0
#endif

/** \file dvect.c
    
\brief Contains simple functions based on the Dense_Vector struct.

Summary:
- init_dvect_memory allocates a block of memory.
- give_dvect_memory passes extra memory to the next Dense_Vector

Notes:
- we should add a free function

*/

Dense_Vector init_dvect_memory(unsigned  int n
			       ){
  /** This routine gets a large amount of memory for Dense_Vector's,
      and gives it to a single Dense_Vector to hold. This special
      Dense_Vector acts as a pointer to this block of memory, and
      also keeps track of how large this memory is.

      INPUT:  n -- the amount of memory to get

      OUPUT: returns a Dense_Vector with
      - .max_size=n
      - .n_row=0
      - .vector pointing to memory for n REAL's

  */

  Dense_Vector dvect;

#if ENTEREXIT
  printf("enter init_dvect_memory\n");
#endif

  /* set max_size*/
  dvect.max_size=n;

  /* empty vector */
  dvect.n_row=0;

  /* main memory for the vector*/
  dvect.vector=(REAL *)calloc(n,sizeof(REAL));
  assert(dvect.vector != NULL);

#if ENTEREXIT
  printf("return from init_dvect_memory\n");
#endif

  return dvect;
}


void give_dvect_memory(Dense_Vector *current,
		       Dense_Vector *previous
		       ){
  /** This routine sets up memory for the next Dense_Vector we wish to use.
      It looks at the previous Dense_Vector we used, and increments fields

      INPUT: previous -- the last Dense_Vector we used

      OUTPUT: current -- the Dense_Vector we wish to use next
                        We align current->vector to the first space not used
			by previous->vector, and set current->max_size to 
			account for the amount used in previous.
  */
#if ENTEREXIT
  printf("entering give_dvect_memory\n");
#endif

  current->n_row=0; /* not used */

  /* align vector to next unused memory */
  current->vector=previous->vector + previous->n_row;

  /* adjust max_size to account for memory used */
  current->max_size=previous->max_size - previous->n_row;

#if ENTEREXIT
  printf("exiting give_dvect_memory\n");
#endif
}


