/*
 * Copyright (c) 2000, 2012 Martin J. Mohlenkamp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "libftsh.h"

#ifndef ENTEREXIT
#define ENTEREXIT 0
#endif


/*************************************************************************/
void ftgaqd(int *nlat,
	    REAL *theta,
	    REAL *wts,
	    REAL *work,
	    int *lwork,
	    int *ierror
	    ){
  /** This routine emulates gadq in the SPHEREPACK library

  INPUTS:
  - nlat --   the number of gaussian colatitudes in the interval (0,pi)
    (between the two poles).  nlat must be greater than zero.
  -  work --  a temporary work space NOT USED
  - lwork -- the length of the work space NOT USED

  OUTPUTS:
  - theta --  a REAL vector of length nlat containing the
    nlat gaussian colatitudes on the sphere in increasing 
    radians in the interval (0,pi).
    Memory must already be allocated
  - wts --   a REAL vector of length nlat containing the nlat 
    gaussian weights.Memory must already be allocated
  - ierror -- given value
    + 0 no errors
    + 2 if nlat<=0

  NOTES:
    - the precision to which to compute the gaussian nodes is hard-wired
      at 1e-15.
    - If I make it ftgaqd_ can I call it from fortran??
    - Where is this routine used??
    */

#if ENTEREXIT
  printf("entering ftgadq\n");
#endif

  /* parse for error */
  if(*nlat > 0)
    ierror=0;
  else{
    *ierror=2;
    return;
  }

  /* call the regular node/weight maker */
  mk_gnode_gweight(theta,wts,1e-15,*nlat);

#if ENTEREXIT
  printf("exiting ftgadq\n");
#endif
}
/*************************************************************************/


#if 0 /* SPHEREPACK routine we are emulating */
c
c  . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c  .                                                             .
c  .                  copyright (c) 1998 by ucar                 .
c  .                                                             .
c  .       university corporation for atmospheric research       .
c  .                                                             .
c  .                      all rights reserved                    .
c  .                                                             .
c  .                                                             .
c  .                         spherepack3.0                       .
c  .                                                             .
c  . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c
c
c
c ... file gaqd.f
c
c     this file includes documentation and code for subrooutine gaqd.
c
c ... required files
c
c     none
c
c     subroutine gaqd(nlat,theta,wts,dwork,ldwork,ierror)
c
c     subroutine gaqd computes the nlat gaussian colatitudes and weights
c     in double precision. the colatitudes are in radians and lie in the
c     in the interval (0,pi).
c
c     input parameters
c
c     nlat    the number of gaussian colatitudes in the interval (0,pi)
c             (between the two poles).  nlat must be greater than zero.
c
c     dwork   a temporary work space
c
c     ldwork  the length of the work space  in the routine calling gaqd
c             ldwork must be at least nlat*(nlat+2).
c
c     output parameters
c
c     theta   a double precision vector of length nlat containing the
c             nlat gaussian colatitudes on the sphere in increasing radians
c             in the interval (o,pi).
c
c     wts     a double precision vector of length nlat containing the
c             nlat gaussian weights.
c
c     ierror = 0 no errors
c            = 1 if ldwork.lt.nlat*(nlat+2)
c            = 2 if nlat.le.0
c            = 3 if unable to compute gaussian points
c                (failure in in eigenvalue routine)
c
c  *****************************************************************
#endif
