/*
 * Copyright (c) 2000, 2012 Martin J. Mohlenkamp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "libftsh.h"

#ifndef ENTEREXIT
#define ENTEREXIT 0
#endif

/*************************************************************************/
void gnode_to_Pmninit(REAL ***sofar,
		      REAL **ex,
		      REAL **why,
		      int **size,
		      REAL *gnode,
		      int hemipts
		      ){
  /** Converts gnodes to the initialization needed for next_rtsinPmncos.

  Takes the place of the initialization step in
  next_rtsinPmncos(....,-1)

  INPUTS: 
  - sofar-- a 2xhalfpts matrix used to store intermediate results
  - ex --  an array of length halfpts containing 
  - why--  an array of length halfpts containing 
  - size--an array of integers of length halfpoints
    used to store the exponent of the power of 2 removed
    to stabilize our function values
  - hemipts-- the number of point in [0,PI/2].  
    We use parity for [PI/2,PI]

  OUTPUTS : This function  creates the arrays above:
  - ex - will be filled in with cos(gnode[i]).
  - why -- will be filled in with sin(gnode[i]).

  */


  int i,index,lowersign;
  REAL lower,upper,mid;

#if ENTEREXIT
  printf("entering gnode_to_Pmninit\n");
#endif

  /* Initialize memory **********/
  
  (*sofar)=(REAL **)calloc(2,sizeof(REAL *));
  (*sofar)[0]=(REAL *)calloc(2*hemipts,sizeof(REAL));
  (*sofar)[1]=(*sofar)[0]+hemipts;
  (*ex)=(REAL *)calloc(hemipts,sizeof(REAL));
  (*why)=(REAL *)calloc(hemipts,sizeof(REAL));
  (*size)=(int *)calloc(hemipts,sizeof(int));

  /* now initialize ex=cos(gnod) and why=sin(gnode)*/

  for(i=0;i<hemipts;i++){
    (*ex)[i]=cos(gnode[i]);
    (*why)[i]=sin(gnode[i]);
  }

#if ENTEREXIT
  printf("exiting gnode_to_Pmninit\n");
#endif

}
/*************************************************************************/

